@@ -60,9 +60,8 @@ if __name__ == '__main__':
                            log_refresh_rate=params['log_refresh_rate'], train_batch_size=params['train_batch_size'],
                            test_batch_size=params['test_batch_size'], num_workers=params['num_workers'],
                            device=params['gpus'][0], transform=transform, feature_maps=params['fm'],
-                           levels=params['levels'], dropout_enc=params['dropout'], dropout_dec=params['dropout'],
-                           norm=params['norm'], activation=params['activation'], coi=params['coi'],
-                           loss_fn=params['loss'])
+                           levels=params['levels'], dropout=params['dropout'], norm=params['norm'],
+                           activation=params['activation'], coi=params['coi'], loss_fn=params['loss'])
 
     """
         Perform cross validation grid search
