@@ -1,4 +1,6 @@
 
+import warnings
+
 from neuralnets.util.tools import sample_unlabeled_input, sample_labeled_input, normalize
 from neuralnets.util.augmentation import split_segmentation_transforms
 from neuralnets.data.base import *
@@ -164,10 +166,18 @@ class LabeledVolumeDataset(VolumeDataset):
     """
     Dataset for pixel-wise labeled volumes
 
-    :param data: path to the dataset or a 3D volume that has already been loaded
-    :param labels: path to the labels or a 3D volume that has already been loaded
+    :param data: input data, possible formats
+            - path to the dataset (string)
+            - preloaded 3D volume (numpy array)
+            - list of paths to multiple datasets (list of strings)
+            - list of preloaded 3D volumes (list of numpy arrays)
+    :param labels: path to the labels or a 3D volume that has already been loaded, possible formats:
+            - path to the dataset (string)
+            - preloaded 3D volume (numpy array)
+            - list of paths to multiple datasets (list of strings)
+            - list of preloaded 3D volumes (list of numpy arrays)
     :param input_shape: 3-tuple that specifies the input shape for sampling
-    :param optional scaling: tuple used for rescaling the data, or None
+    :param optional scaling: 3-tuple used for rescaling the data, or a list of 3-tuples in case of multiple datasets
     :param optional len_epoch: number of iterations for one epoch
     :param optional type: type of the volume file (tif2d, tif3d, tifseq, hdf5, png or pngseq)
     :param optional coi: list or sequence of the classes of interest
@@ -178,52 +188,69 @@ class LabeledVolumeDataset(VolumeDataset):
     :param optional label_dtype: type of the labels (typically uint8)
     :param optional norm_type: type of the normalization (unit, z or minmax)
     :param optional transform: augmenter object
-    :param optional range_split: range of slices (start, stop) to select (normalized between 0 and 1)
-    :param optional range_dir: orientation of the slicing
+    :param optional range_split: range of slices (start, stop) to select (normalized between 0 and 1), or a list of
+                                 2-tuples in case of multiple datasets
+    :param optional range_dir: orientation of the slicing or a list of orientations in case of multiple datasets
+    :param optional resolution: list of 3-tuples specifying the pixel resolution of the data
+    :param optional match_resolution_to: match the resolution of all data to a specific dataset
+    :param optional sampling_type: type of sampling in case of multiple datasets
+            - joint: the dataset will generate random samples in each dataset and return all of them
+            - single: the dataset will generate a random sample from a randomly selected dataset and return that
     """
 
     def __init__(self, data, labels, input_shape=None, scaling=None, len_epoch=None, type='tif3d', coi=(0, 1),
                  in_channels=1, orientations=(0,), batch_size=1, data_dtype='uint8', label_dtype='uint8',
-                 norm_type='unit', transform=None, range_split=None, range_dir=None):
+                 norm_type='unit', transform=None, range_split=None, range_dir=None, resolution=None,
+                 match_resolution_to=None, sampling_type='joint'):
         super().__init__(data, input_shape, scaling=scaling, len_epoch=len_epoch, type=type,
                          in_channels=in_channels, orientations=orientations, batch_size=batch_size, dtype=data_dtype,
-                         norm_type=norm_type, range_split=range_split, range_dir=range_dir)
+                         norm_type=norm_type, range_split=range_split, range_dir=range_dir, resolution=resolution,
+                         match_resolution_to=match_resolution_to, sampling_type=sampling_type)
 
-        if isinstance(labels, str):
-            self.labels_path = labels
-            # load labels
-            self.labels = read_volume(labels, type=type, dtype=label_dtype)
+        if isinstance(labels, str) or isinstance(labels, np.ndarray):
+            self.labels = [load_data(labels, data_type=type, dtype=label_dtype)]
+        elif isinstance(labels, list) or isinstance(labels, tuple):  # list of data
+            self.labels = []
+            for labels_i in labels:
+                self.labels.append(load_data(labels_i, data_type=type, dtype=label_dtype))
         else:
-            self.labels = labels
+            raise ValueError('LabeledVolumeDataset requires labels in str, np.ndarray or list format')
         self.coi = coi
         self.transform = transform
         if transform is not None:
             self.shared_transform, self.x_transform, self.y_transform = split_segmentation_transforms(transform)
 
         # select a subset of slices of the data
-        self.labels = slice_subset(self.labels, range_split, range_dir)
+        for i in range(len(self.labels)):
+            if isinstance(range_split, list) and isinstance(range_dir, list):
+                self.labels[i] = slice_subset(self.labels[i], range_split[i], range_dir[i])
+            else:
+                self.labels[i] = slice_subset(self.labels[i], range_split, range_dir)
 
         # rescale the dataset if necessary
-        if scaling is not None:
-            target_size = np.asarray(np.multiply(self.labels.shape, scaling), dtype=int)
-            self.labels = F.interpolate(torch.Tensor(self.labels[np.newaxis, np.newaxis, ...]), size=tuple(target_size),
-                                        mode='area')[0, 0, ...].numpy()
+        for i in range(len(self.labels)):
+            if (self.resolution is not None and self.match_resolution_to is not None) or self.scaling is not None:
+                if self.resolution is not None and self.match_resolution_to is not None:
+                    scale_factor = np.divide(self.labels[self.match_resolution_to].shape, self.labels[i].shape)
+                else:
+                    if isinstance(self.scaling, list):
+                        scale_factor = self.scaling[i]
+                    else:
+                        scale_factor = self.scaling
+                labels_it = torch.Tensor(self.labels[i]).unsqueeze(0).unsqueeze(0)
+                self.labels[i] = F.interpolate(labels_it, scale_factor=scale_factor, mode='bilinear')[0, 0, ...].numpy()
 
         # relabel classes of interest
-        self.labels = _map_cois(self.labels, self.coi)
+        self.labels = [_map_cois(l, self.coi) for l in self.labels]
 
-    def __getitem__(self, i, attempt=0):
-
-        # reorient when we start a new batch
-        if i % self.batch_size == 0:
-            self._select_orientation()
+    def _sample(self, data_index):
 
         # get shape of sample
-        input_shape = _validate_shape(self.input_shape, self.data.shape, in_channels=self.in_channels,
+        input_shape = _validate_shape(self.input_shape, self.data[data_index].shape, in_channels=self.in_channels,
                                       orientation=self.orientation)
 
         # get random sample
-        x, y = sample_labeled_input(self.data, self.labels, input_shape)
+        x, y = sample_labeled_input(self.data[data_index], self.labels[data_index], input_shape)
         x = normalize(x, type=self.norm_type)
         y = y.astype(float)
 
@@ -251,13 +278,57 @@ class LabeledVolumeDataset(VolumeDataset):
         x = torch.from_numpy(x).float()
         y = torch.from_numpy(y).long()
 
-        # make sure we have at least one labeled pixel in the sample, otherwise processing is useless
-        if len(np.intersect1d(torch.unique(y).numpy(), self.coi)) == 0:
-            if attempt < MAX_SAMPLING_ATTEMPTS:
-                x, y = self.__getitem__(i, attempt=attempt + 1)
-            # else:
-                # warnings.warn("No labeled pixels found after %d sampling attempts! ")
+        return x, y
 
+    def __getitem__(self, i, attempt=0):
+
+        # reorient when we start a new batch
+        if i % self.batch_size == 0:
+            self._select_orientation()
+
+        if self.sampling_type == 'single':
+
+            # randomly select a dataset
+            r = np.random.randint(len(self.data))
+
+            # select a sample from dataset r
+            x, y = self._sample(r)
+
+            # make sure we have at least one labeled pixel in the sample, otherwise processing is useless
+            if len(np.intersect1d(torch.unique(y).numpy(), self.coi)) == 0 and not self.warned:
+                if attempt < MAX_SAMPLING_ATTEMPTS:
+                    x, y = self.__getitem__(i, attempt=attempt + 1)
+                else:
+                    warnings.warn("No labeled pixels found after %d sampling attempts! " % attempt)
+                    self.warned = True
+
+        else:  # joint sampling
+
+            xs = []
+            ys = []
+
+            for r in range(len(self.data)):
+
+                # select a sample from dataset r
+                x, y = self._sample(r)
+
+                xs.append(x)
+                ys.append(y)
+
+            if len(self.data) == 1:
+                x = xs[0]
+                y = ys[0]
+            else:
+                x = xs
+                y = ys
+
+            # make sure we have at least one labeled pixel in the sample, otherwise processing is useless
+            if np.sum([len(np.intersect1d(torch.unique(y_).numpy(), self.coi)) for y_ in ys]) == 0 and not self.warned:
+                if attempt < MAX_SAMPLING_ATTEMPTS:
+                    x, y = self.__getitem__(i, attempt=attempt + 1)
+                else:
+                    warnings.warn("No labeled pixels found after %d sampling attempts! " % attempt)
+                    self.warned = True
 
         # return sample
         return x, y
@@ -267,9 +338,13 @@ class UnlabeledVolumeDataset(VolumeDataset):
     """
     Dataset for unlabeled volumes
 
-    :param data_path: path to the dataset
+    :param data: input data, possible formats
+            - path to the dataset (string)
+            - preloaded 3D volume (numpy array)
+            - list of paths to multiple datasets (list of strings)
+            - list of preloaded 3D volumes (list of numpy arrays)
     :param input_shape: 3-tuple that specifies the input shape for sampling
-    :param optional scaling: tuple used for rescaling the data, or None
+    :param optional scaling: 3-tuple used for rescaling the data, or a list of 3-tuples in case of multiple datasets
     :param optional len_epoch: number of iterations for one epoch
     :param optional type: type of the volume file (tif2d, tif3d, tifseq, hdf5, png or pngseq)
     :param optional in_channels: amount of subsequent slices to be sampled (only for 2D sampling)
@@ -277,15 +352,52 @@ class UnlabeledVolumeDataset(VolumeDataset):
     :param optional batch_size: size of the sampling batch
     :param optional dtype: type of the data (typically uint8)
     :param optional norm_type: type of the normalization (unit, z or minmax)
+    :param optional transform: augmenter object
+    :param optional range_split: range of slices (start, stop) to select (normalized between 0 and 1), or a list of
+                                 2-tuples in case of multiple datasets
+    :param optional range_dir: orientation of the slicing or a list of orientations in case of multiple datasets
+    :param optional resolution: list of 3-tuples specifying the pixel resolution of the data
+    :param optional match_resolution_to: match the resolution of all data to a specific dataset
+    :param optional sampling_type: type of sampling in case of multiple datasets
+            - joint: the dataset will generate random samples in each dataset and return all of them
+            - single: the dataset will generate a random sample from a randomly selected dataset and return that
     """
 
-    def __init__(self, data_path, input_shape=None, scaling=None, len_epoch=1000, type='tif3d', in_channels=1,
-                 orientations=(0,), batch_size=1, dtype='uint8', norm_type='unit'):
-        super().__init__(data_path, input_shape, scaling=scaling, len_epoch=len_epoch, type=type,
+    def __init__(self, data, input_shape=None, scaling=None, len_epoch=None, type='tif3d', in_channels=1,
+                 orientations=(0,), batch_size=1, dtype='uint8', norm_type='unit', transform=None, range_split=None,
+                 range_dir=None, resolution=None, match_resolution_to=None, sampling_type='joint'):
+        super().__init__(data, input_shape, scaling=scaling, len_epoch=len_epoch, type=type,
                          in_channels=in_channels, orientations=orientations, batch_size=batch_size, dtype=dtype,
-                         norm_type=norm_type)
+                         norm_type=norm_type, range_split=range_split, range_dir=range_dir, resolution=resolution,
+                         match_resolution_to=match_resolution_to, sampling_type=sampling_type)
 
-        self.mu, self.std = self._get_stats()
+        self.transform = transform
+
+    def _sample(self, data_index):
+
+        # get shape of sample
+        input_shape = _validate_shape(self.input_shape, self.data[data_index].shape, in_channels=self.in_channels,
+                                      orientation=self.orientation)
+
+        # get random sample
+        x = sample_unlabeled_input(self.data[data_index], input_shape)
+        x = normalize(x, type=self.norm_type)
+
+        # reorient sample
+        x = _orient(x, orientation=self.orientation)
+
+        # add channel axis if the data is 3D
+        if self.input_shape[0] > 1:
+            x = x[np.newaxis, ...]
+
+        # augment sample
+        if self.transform is not None:
+            x = self.transform(x)
+
+        # transform to tensors
+        x = torch.from_numpy(x).float()
+
+        return x
 
     def __getitem__(self, i):
 
@@ -293,32 +405,50 @@ class UnlabeledVolumeDataset(VolumeDataset):
         if i % self.batch_size == 0:
             self._select_orientation()
 
-        # get shape of sample
-        input_shape = _validate_shape(self.input_shape, self.data.shape, in_channels=self.in_channels,
-                                      orientation=self.orientation)
+        if self.sampling_type == 'single':
 
-        # get random sample
-        input = sample_unlabeled_input(self.data, input_shape)
-        input = normalize(input, type=self.norm_type)
+            # randomly select a dataset
+            r = np.random.randint(len(self.data))
 
-        # reorient sample
-        input = _orient(input, orientation=self.orientation)
+            # select a sample from dataset r
+            x = self._sample(r)
 
-        if self.input_shape[0] > 1:
-            # add channel axis if the data is 3D
-            return input[np.newaxis, ...]
-        else:
-            return input
+        else:  # joint sampling
+
+            xs = []
+
+            for r in range(len(self.data)):
+
+                # select a sample from dataset r
+                x = self._sample(r)
+
+                xs.append(x)
+
+            if len(self.data) == 1:
+                x = xs[0]
+            else:
+                x = xs
+
+        # return sample
+        return x
 
 
 class LabeledSlidingWindowDataset(SlidingWindowDataset):
     """
     Dataset for pixel-wise labeled volumes with a sliding window
 
-    :param data: path to the dataset
-    :param labels: path to the labels
+    :param data: input data, possible formats
+            - path to the dataset (string)
+            - preloaded 3D volume (numpy array)
+            - list of paths to multiple datasets (list of strings)
+            - list of preloaded 3D volumes (list of numpy arrays)
+    :param labels: path to the labels or a 3D volume that has already been loaded, possible formats:
+            - path to the dataset (string)
+            - preloaded 3D volume (numpy array)
+            - list of paths to multiple datasets (list of strings)
+            - list of preloaded 3D volumes (list of numpy arrays)
     :param input_shape: 3-tuple that specifies the input shape for sampling
-    :param optional scaling: tuple used for rescaling the data, or None
+    :param optional scaling: 3-tuple used for rescaling the data, or a list of 3-tuples in case of multiple datasets
     :param optional type: type of the volume file (tif2d, tif3d, tifseq, hdf5, png or pngseq)
     :param optional in_channels: amount of subsequent slices to be sampled (only for 2D sampling)
     :param optional orientations: list of orientations for sampling
@@ -328,59 +458,85 @@ class LabeledSlidingWindowDataset(SlidingWindowDataset):
     :param optional label_dtype: type of the labels (typically uint8)
     :param optional norm_type: type of the normalization (unit, z or minmax)
     :param optional transform: augmenter object
+    :param optional range_split: range of slices (start, stop) to select (normalized between 0 and 1), or a list of
+                                 2-tuples in case of multiple datasets
+    :param optional range_dir: orientation of the slicing or a list of orientations in case of multiple datasets
+    :param optional resolution: list of 3-tuples specifying the pixel resolution of the data
+    :param optional match_resolution_to: match the resolution of all data to a specific dataset
     """
 
     def __init__(self, data, labels, input_shape=None, scaling=None, type='tif3d', in_channels=1, orientations=(0,),
                  coi=(0, 1), batch_size=1, data_dtype='uint8', label_dtype='uint8', norm_type='unit', transform=None,
-                 range_split=None, range_dir=None):
+                 range_split=None, range_dir=None, resolution=None, match_resolution_to=None):
         super().__init__(data, input_shape, scaling=scaling, type=type, in_channels=in_channels,
                          orientations=orientations, batch_size=batch_size, dtype=data_dtype, norm_type=norm_type,
-                         range_split=range_split, range_dir=range_dir)
+                         range_split=range_split, range_dir=range_dir, resolution=resolution,
+                         match_resolution_to=match_resolution_to)
 
-        if isinstance(labels, str):
-            self.labels_path = labels
-            # load labels
-            self.labels = read_volume(labels, type=type, dtype=label_dtype)
+        if isinstance(labels, str) or isinstance(labels, np.ndarray):
+            self.labels = [load_data(labels, data_type=type, dtype=label_dtype)]
+        elif isinstance(labels, list) or isinstance(labels, tuple):  # list of data
+            self.labels = []
+            for labels_i in labels:
+                self.labels.append(load_data(labels_i, data_type=type, dtype=label_dtype))
         else:
-            self.labels = labels
+            raise ValueError('LabeledSlidingWindowDataset requires labels in str, np.ndarray or list format')
         self.coi = coi
         self.transform = transform
         if transform is not None:
             self.shared_transform, self.x_transform, self.y_transform = split_segmentation_transforms(transform)
 
         # select a subset of slices of the data
-        self.labels = slice_subset(self.labels, range_split, range_dir)
+        for i in range(len(self.labels)):
+            if isinstance(range_split, list) and isinstance(range_dir, list):
+                self.labels[i] = slice_subset(self.labels[i], range_split[i], range_dir[i])
+            else:
+                self.labels[i] = slice_subset(self.labels[i], range_split, range_dir)
 
         # rescale the dataset if necessary
-        if scaling is not None:
-            target_size = np.asarray(np.multiply(self.labels.shape, scaling), dtype=int)
-            self.labels = F.interpolate(torch.Tensor(self.labels[np.newaxis, np.newaxis, ...]), size=tuple(target_size),
-                                        mode='area')[0, 0, ...].numpy()
+        for i in range(len(self.labels)):
+            if (self.resolution is not None and self.match_resolution_to is not None) or self.scaling is not None:
+                if self.resolution is not None and self.match_resolution_to is not None:
+                    scale_factor = np.divide(self.labels[self.match_resolution_to].shape, self.labels[i].shape)
+                else:
+                    if isinstance(self.scaling, list):
+                        scale_factor = self.scaling[i]
+                    else:
+                        scale_factor = self.scaling
+                labels_it = torch.Tensor(self.labels[i]).unsqueeze(0).unsqueeze(0)
+                self.labels[i] = F.interpolate(labels_it, scale_factor=scale_factor, mode='bilinear')[0, 0, ...].numpy()
 
         # pad data so that the dimensions are a multiple of the inputs shapes
-        self.labels = pad2multiple(self.labels, input_shape, value=255)
+        self.labels = [pad2multiple(l, input_shape, value=255) for l in self.labels]
 
         # pad data so that additional channels can be sampled
-        self.labels = pad_channels(self.labels, in_channels=in_channels, orientations=self.orientations)
+        self.labels = [pad_channels(l, in_channels=in_channels, orientations=self.orientations) for l in self.labels]
 
         # relabel classes of interest
-        self.labels = _map_cois(self.labels, self.coi)
+        self.labels = [_map_cois(l, self.coi) for l in self.labels]
 
     def __getitem__(self, i):
 
+        # find dataset index
+        r = 0
+        szs = self.n_samples_dim.prod(axis=1)
+        while szs[:r + 1].sum() <= i:
+            r += 1
+
         # get spatial location
-        iz = i // (self.n_samples_dim[1] * self.n_samples_dim[2])
-        iy = (i - iz * self.n_samples_dim[1] * self.n_samples_dim[2]) // self.n_samples_dim[2]
-        ix = i - iz * self.n_samples_dim[1] * self.n_samples_dim[2] - iy * self.n_samples_dim[2]
+        j = i - szs[:r].sum()
+        iz = j // (self.n_samples_dim[r, 1] * self.n_samples_dim[r, 2])
+        iy = (j - iz * self.n_samples_dim[r, 1] * self.n_samples_dim[r, 2]) // self.n_samples_dim[r, 2]
+        ix = j - iz * self.n_samples_dim[r, 1] * self.n_samples_dim[r, 2] - iy * self.n_samples_dim[r, 2]
         pz = self.input_shape[0] * iz
         py = self.input_shape[1] * iy
         px = self.input_shape[2] * ix
 
         # get shape of sample
-        input_shape = _validate_shape(self.input_shape, self.data.shape, in_channels=self.in_channels)
+        input_shape = _validate_shape(self.input_shape, self.data[r].shape, in_channels=self.in_channels)
 
         # get sample
-        x, y = sample_labeled_input(self.data, self.labels, input_shape, zyx=(pz, py, px))
+        x, y = sample_labeled_input(self.data[r], self.labels[r], input_shape, zyx=(pz, py, px))
         x = normalize(x, type=self.norm_type)
         y = y.astype(float)
 
@@ -404,135 +560,10 @@ class LabeledSlidingWindowDataset(SlidingWindowDataset):
         x = torch.from_numpy(x).float()
         y = torch.from_numpy(y).long()
 
-        # # make sure we have at least one labeled pixel in the sample, otherwise processing is useless
-        # if len(np.intersect1d(torch.unique(y).numpy(), self.coi)) == 0:
-        #     warnings.warn("No labeled pixels found! ")
+        # make sure we have at least one labeled pixel in the sample, otherwise processing is useless
+        if len(np.intersect1d(torch.unique(y).numpy(), np.arange(len(self.coi)))) == 0 and not self.warned:
+            warnings.warn("No labeled pixels found! ")
+            self.warned = True
 
         # return sample
         return x, y
-
-
-class LabeledMultiVolumeDataset(MultiVolumeDataset):
-    """
-    Dataset for multiple pixel-wise labeled volumes
-
-    :param data_path: list of paths to the datasets
-    :param data_path: list of paths to the labels
-    :param input_shape: 3-tuple that specifies the input shape for sampling
-    :param optional scaling: tuple used for rescaling the data, or None
-    :param optional len_epoch: number of iterations for one epoch
-    :param optional types: list of types of the volume files (tif2d, tif3d, tifseq, hdf5, png or pngseq)
-    :param optional sampling_mode: allow for uniform balance in sampling or not ("uniform" or "random")
-    :param optional coi: list or sequence of the classes of interest
-    :param optional in_channels: amount of subsequent slices to be sampled (only for 2D sampling)
-    :param optional orientations: list of orientations for sampling
-    :param optional batch_size: size of the sampling batch
-    :param optional data_dtype: type of the data (typically uint8)
-    :param optional label_dtype: type of the labels (typically uint8)
-    :param optional norm_type: type of the normalization (unit, z or minmax)
-    """
-
-    def __init__(self, data_path, label_path, input_shape=None, scaling=None, len_epoch=1000, types=['tif3d'],
-                 sampling_mode='uniform', coi=(0, 1), in_channels=1, orientations=(0,), batch_size=1,
-                 data_dtype='uint8', label_dtype='uint8', norm_type='unit'):
-        super().__init__(data_path, input_shape, scaling=scaling, len_epoch=len_epoch, types=types,
-                         sampling_mode=sampling_mode, in_channels=in_channels, orientations=orientations,
-                         batch_size=batch_size, dtype=data_dtype, norm_type=norm_type)
-
-        self.label_path = label_path
-        self.coi = coi
-
-        # load the data
-        self.labels = []
-        for k, path in enumerate(label_path):
-            labels = read_volume(path, type=types[k], dtype=label_dtype)
-
-            # rescale the dataset if necessary
-            if scaling is not None:
-                target_size = np.asarray(np.multiply(labels.shape, scaling[k]), dtype=int)
-                labels = F.interpolate(torch.Tensor(labels[np.newaxis, np.newaxis, ...]), size=tuple(target_size),
-                                       mode='area')[0, 0, ...].numpy()
-
-            self.labels.append(labels)
-
-        self.mu, self.std = self._get_stats()
-
-    def __getitem__(self, i):
-
-        # reorient when we start a new batch
-        if i % self.batch_size == 0:
-            self._select_dataset()
-            self._select_orientation()
-
-        # get shape of sample
-        input_shape = _validate_shape(self.input_shape, self.data[self.k].shape, in_channels=self.in_channels,
-                                      orientation=self.orientation)
-
-        # get random sample
-        input, target = sample_labeled_input(self.data[self.k], self.labels[self.k], input_shape)
-        input = normalize(input, type=self.norm_type)
-
-        # reorient sample
-        input = _orient(input, orientation=self.orientation)
-        target = _orient(target, orientation=self.orientation)
-
-        if self.input_shape[0] > 1:
-            # add channel axis if the data is 3D
-            input, target = input[np.newaxis, ...], target[np.newaxis, ...]
-
-        if len(np.intersect1d(np.unique(target),
-                              self.coi)) == 0:  # make sure we have at least one labeled pixel in the sample, otherwise processing is useless
-            return self.__getitem__(i)
-        else:
-            return input, target
-
-
-class UnlabeledMultiVolumeDataset(MultiVolumeDataset):
-    """
-    Dataset for multiple unlabeled volumes
-
-    :param data_path: list of paths to the datasets
-    :param input_shape: 3-tuple that specifies the input shape for sampling
-    :param optional scaling: tuple used for rescaling the data, or None
-    :param optional len_epoch: number of iterations for one epoch
-    :param optional types: list of types of the volume files (tif2d, tif3d, tifseq, hdf5, png or pngseq)
-    :param optional in_channels: amount of subsequent slices to be sampled (only for 2D sampling)
-    :param optional sampling_mode: allow for uniform balance in sampling or not ("uniform" or "random")
-    :param optional orientations: list of orientations for sampling
-    :param optional batch_size: size of the sampling batch
-    :param optional dtype: type of the data (typically uint8)
-    :param optional norm_type: type of the normalization (unit, z or minmax)
-    """
-
-    def __init__(self, data_path, input_shape=None, scaling=None, len_epoch=1000, types='tif3d',
-                 sampling_mode='uniform', in_channels=1, orientations=(0,), batch_size=1, dtype='uint8',
-                 norm_type='unit'):
-        super().__init__(data_path, input_shape, scaling=scaling, len_epoch=len_epoch, types=types,
-                         sampling_mode=sampling_mode, in_channels=in_channels, orientations=orientations,
-                         batch_size=batch_size, dtype=dtype, norm_type=norm_type)
-
-        self.mu, self.std = self._get_stats()
-
-    def __getitem__(self, i):
-
-        # reorient when we start a new batch
-        if i % self.batch_size == 0:
-            self._select_dataset()
-            self._select_orientation()
-
-        # get shape of sample
-        input_shape = _validate_shape(self.input_shape, self.data[self.k].shape, in_channels=self.in_channels,
-                                      orientation=self.orientation)
-
-        # get random sample
-        input = sample_unlabeled_input(self.data[self.k], input_shape)
-        input = normalize(input, type=self.norm_type)
-
-        # reorient sample
-        input = _orient(input, orientation=self.orientation)
-
-        if input.shape[0] > 1:
-            # add channel axis if the data is 3D
-            return input[np.newaxis, ...]
-        else:
-            return input
